
# q-expansion of newform 3850.2.a.s, downloaded from the LMFDB on 07 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3850
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [0], [0], [-1], [-1], [6], [2], [-4], [4], [6], [0], [2], [-6], [4], [-4], [2], [12], [-2], [8], [-8], [10], [-8], [12], [10], [6], [14], [4], [-4], [14], [6], [8], [-12], [-18], [-20], [6], [8], [2], [-8], [0], [14], [20], [6], [8], [-6], [-18], [-24], [4], [4], [-4], [6], [10], [-24], [2], [20], [-18], [-16], [-2], [0], [-26], [-30], [20], [14], [20], [0], [-10], [-14], [-28], [-22], [28], [-34], [14], [-16], [-20], [-34], [-12], [28], [6], [34], [34], [34], [12], [-26], [8], [6], [-24], [-16], [2], [26], [6], [-20], [0], [-24], [28], [20], [28], [8], [-2], [-22], [12], [-18], [-36], [6], [4], [42], [4], [38], [-8], [-30], [-40], [26], [-40], [-34], [-34], [4], [8], [-30], [-32], [12], [42], [36], [22], [10], [14], [-24], [-36], [-42], [22], [-40], [12], [46], [-12], [0], [-32], [-6], [-14], [-14], [-30], [-28], [34], [-46], [52], [30], [-4], [12], [30], [-40], [6], [42], [-44], [52], [-10], [2], [-56], [48], [-32], [-40], [-40], [-30], [50], [38], [-24], [-22], [32], [-52], [54], [4], [-40], [-10], [26], [50], [-20], [-2], [-24], [-6], [-24], [26], [20], [-58], [40], [-10], [-20], [12], [10], [-30], [-36], [22], [54], [-56], [10], [8], [38], [36], [-28], [-2], [60], [6], [58], [58], [-6], [-32], [54], [40], [-46], [-46], [36], [18], [-32], [-60], [34], [44], [-62], [-10], [-36], [8], [0], [-38], [32], [10], [52], [10], [-2], [-48], [50], [4], [-28], [-58], [-50], [-24], [-32], [12], [18], [-28], [72], [66], [0], [48], [-30], [38], [-44], [72], [72], [28], [-76], [-18], [42], [48], [28], [60], [28], [24], [-50], [26], [36], [26], [-34], [-44], [-10], [-76], [26], [-62], [24], [-4], [22], [-42], [-2], [-60], [30], [10], [20], [-74], [78], [8], [30], [0], [42], [12], [16], [70], [50], [36], [-8], [32], [-24], [-18], [-44], [8], [-58], [-58], [0], [-54], [78], [8], [-54], [60], [78], [-74], [8], [-46], [-36], [-12], [34], [22], [16], [-48], [20], [-34], [-48], [14], [-24], [30], [80], [-10], [2], [80], [16], [26], [12], [0], [86], [-30], [-44], [30], [78], [-4], [-38], [-78], [20], [-56], [-48], [78], [-26], [66], [-24], [84], [28], [-48], [-34], [-6], [-22], [48], [-22], [38], [-50], [32], [-86], [12], [-58], [-72], [88], [-62], [-20], [22], [-2], [-16], [14], [42], [48], [12], [2], [-36], [22], [-30], [68], [-12], [56], [-34], [-2], [-16], [-30], [-12], [20], [24], [6], [-64], [-14], [28], [-64], [-70], [58], [2], [-34], [6], [16], [-22], [44], [36], [0], [-54], [-20], [-12], [-62], [-62], [36], [-64], [88], [-38], [0], [42], [-68], [62], [6], [-90], [32], [6], [-18], [-40], [34], [82], [24], [20], [58], [-42], [-16], [-100], [-14], [-18], [16], [-4], [-66], [24], [94], [-82], [-36], [-44], [-58], [58], [-72], [42], [-92], [-16]]
